# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that defines the behavior of the metadata
label on the player.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class Label(object):
	"""
	This class defines the behavior of the metadata label on the player.
	"""

	def __init__(self, manager, player):
		"""
		Initialize object.

		@param self: Reference to the Label instance.
		@type self: A Label object.

		@param player: Reference to the Player.
		@type player: A Player object.
		"""
		self.__init_attributes(manager, player)
		self.__update_label()
		self.__signal_id_1 = player.connect("stream-update", self.__stream_update_cb)
		self.__signal_id_2 = player.connect("streamer-is-busy", self.__streamer_is_busy_cb)
		self.__signal_id_3 = player.connect("streamer-is-stopped", self.__streamer_is_stopped_cb)
		self.__signal_id_4 = player.connect("playlist-error", self.__playlist_error_cb)
		self.__signal_id_5 = player.connect("stream-error", self.__stream_error_cb)
		self.__signal_id_6 = manager.connect("destroy", self.__destroy_cb)
		self.__signal_id_7 = player.connect("stream-message", self.__stream_message_cb)

	def __init_attributes(self, manager, player):
		"""
		Initialize data attributes.

		@param self: Reference to the Label instance.
		@type self: A Label object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__player = player
		self.__string = ""
		self.__manager = manager
		self.__label = manager.glade.get_widget("StatusLabel")
		self.__can_set_label = True
		self.__timeout_id = None
		self.__signal_id_1 = self.__signal_id_2 = None
		self.__signal_id_3 = self.__signal_id_4 = None
		self.__signal_id_5 = None
		return

	def __update_feedback(self, metadata=(), info=False, error=None, buffer_=None):
		"""
		Update label with feeback message.

		@param self: Reference to the Label instance.
		@type self: A Label object.

		@param metadata: Radio station, title of song, time
		@type metadata: A Tuple object.

		@param info: Message shown when getting metadata information.
		@type info: A Boolean object.

		@param error: An error message.
		@type error: A String object.

		@param buffer_: Message shown when buffering streams.
		@type buffer_: A String object.
		"""
		self.__player.response()

		if metadata:
			template = "<b><span foreground='blue'>%s</span></b>\n<i>by</i> %s <span weight='ultralight' size='small' foreground='brown'>%s</span>"
			title, station, time = metadata
			self.__string = template % (title, station, time)
			self.__label.set_property("sensitive", True)
			if self.__can_set_label: self.__label.set_markup(self.__string)
		elif info:
			from Striim.i18n import msg0001
			self.__string = "<i>" + msg0001 + "</i>"
			self.__label.set_property("sensitive", False)
			if self.__can_set_label: self.__label.set_markup(self.__string)
		elif error:
			template = "<b><span foreground='red'>%s</span></b>"
			self.__string = template % (error)
			self.__label.set_property("sensitive", True)
			if self.__can_set_label: self.__label.set_markup(self.__string)
		elif buffer_:
			self.__can_set_label = False
			template = "<i><span foreground='brown'>%s</span></i>"
			string = template % (buffer_)
			self.__label.set_property("sensitive", True)
			self.__label.set_markup(string)
			from gobject import timeout_add
			self.__remove_timer()
			self.__timeout_id = timeout_add(60000, self.__reset_feedback)
		else:
			template = "<b>%s</b>\n<i>by</i> %s <span weight='ultralight' size='small'>0min</span>"
			self.__string = template % ("Artist - Title", "Radio Station")
			self.__label.set_property("sensitive", False)
			self.__label.set_markup(self.__string)
		self.__player.response()
		return

	def __reset_feedback(self):
		"""
		Reset feeback message.

		@param self: Reference to the Label instance.
		@type self: A Label object.

		@return: False to call this function once.
		@rtype: A Boolean object.
		"""
		self.__label.set_markup(self.__string)
		self.__label.set_property("sensitive", True)
		self.__can_set_label = True
		return False

	def __remove_timer(self):
		try:
			from gobject import source_remove
			source_remove(self.__timeout_id)
		except Exception:
			pass
		return

	def __update_label(self):
		"""
		Update the label at startup.

		@param self: Reference to the Label instance.
		@type self: A Label object.
		"""
		if self.__player.is_playing:
			title = self.__player.title
			station = self.__player.station
			time = self.__player.time
			self.__update_feedback(metadata=(title, station, time))
		elif self.__player.is_busy:
			self.__update_feedback(info=True)
		return

	def __stream_update_cb(self, player, dictionary):
		"""
		Handles callback when the "stream-update" signal is emitted.

		@param self: Reference to the Label instance.
		@type self: A Label object.

		@param self: Reference to the Player instance.
		@type self: A Player object.

		@param dictionary: A dictionary containing metadata information.
		@type dictionary: A Dictionary object.
		"""
		try:
			title = dictionary["title"].strip()
			station = dictionary["station"].strip()
			time = dictionary["time"]
			self.__update_feedback(metadata=(title, station, time))
		except KeyError:
			pass
		return

	def __stream_message_cb(self, player, data):
		"""
		Handles callback when the "stream-message" signal is emitted.

		@param self: Reference to the Label instance.
		@type self: A Label object.

		@param player: Reference to the Player instance.
		@type player: A Player object.

		@param data: A string message.
		@type data: A String object.
		"""
		from operator import eq
		if eq(data, "done"):
			self.__remove_timer()
			self.__reset_feedback()
		else:
			self.__update_feedback(buffer_=data)
		return

	def __streamer_is_busy_cb(self, *args):
		"""
		Handles callback when the "streamer-is-busy" signal is emitted.

		@param self: Reference to the Label instance.
		@type self: A Label object.
		"""
		self.__update_feedback(info=True)
		return

	def __streamer_is_stopped_cb(self, *args):
		"""
		Handles callback when the "streamer-is-stopped" signal is emitted.

		@param self: Reference to the Label instance.
		@type self: A Label object.
		"""
		self.__remove_timer()
		self.__update_feedback()
		return

	def __playlist_error_cb(self, *args):
		"""
		Handles callback when the "playlist-error" signal is emitted.

		@param self: Reference to the Label instance.
		@type self: A Label object.
		"""
		self.__remove_timer()
		self.__reset_feedback()
		from i18n import msg0001
		self.__update_feedback(error=msg0001)
		return

	def __stream_error_cb(self, player, message):
		"""
		Handles callback when "stream-error" signal is emitted.

		@param self: Reference to the Label instance.
		@type self: A Label object.

		@param message: An error message.
		@type message: A String object.
		"""
		self.__remove_timer()
		self.__reset_feedback()
		from i18n import msg0002
		message = msg0002 + " " + message
		self.__update_feedback(error=message)
		return

	def __destroy_cb(self, *args):
		"""
		Destroy object.

		@param self: Reference to the Label instance.
		@type self: A Label object.
		"""
		from Striim.utils import delete_attributes, disconnect_signal
		disconnect_signal(self.__signal_id_1, self.__player)
		disconnect_signal(self.__signal_id_2, self.__player)
		disconnect_signal(self.__signal_id_3, self.__player)
		disconnect_signal(self.__signal_id_4, self.__player)
		disconnect_signal(self.__signal_id_5, self.__player)
		disconnect_signal(self.__signal_id_6, self.__manager)
		disconnect_signal(self.__signal_id_7, self.__player)
		self.__label.destroy()
		delete_attributes(self)
		del self
		self = None
		return
